<?php
	
	interface IUsers {
		/**
		 * @param  user the username as supplied by the user in plain text
		 * @param  pwd  the password as supplied by the user in plain text
		 * @return whether the username is valid and the password matches the saved one
		 */
		public function authenticate($user, $pwd);
		
		/**
		 * @return an array containing all registered usernames
		 */
		public function getUsers();
		
		/**
		 * @param  user the username as supplied by the user in plain text
		 * @return whether this user has admin rights
		 */
		public function isAdmin($user);
	}
	
	interface IVote {
		/**
		 * @return the all saved votings as array. Return their respective names as keys and a
		 *         boolean value specifying whether the voting is finished
		 */
		public function getVotingNames();
		
		/**
		 * @param  name name of the voting to return
		 * @return the {@link Voting voting} specified by <code>name</code>
		 * @throws Exception if the voting with the name specified does not exist
		 */
		public function getVoting($name);
		
		/**
		 * @param voting the {@link Voting voting} object to save (may exist already)
		 */
		public function setVoting(&$voting);
		
		/**
		 * @param name the name of the voting to delete
		 * @throws Exception if the voting with the name specified does not exist
		 */
		public function removeVoting($name);
	}
	
	class Voting {
		private $name;
		private $alts;
		private $users;
		private $finished;
		private $multi;
		private $publicly;
		
		/**
		 * @param name the name of this voting as string
		 * @param alts (optional) set of alternatives to choose from as string-array
		 * @param users (optional) initial set of user->votes-relationships as array
		 * @param finished (optional) whether this voting is finished or not
		 * @param multi (optional) whether multiple alternatives may be selected
		 * @param publicly (optional) whether this vote shall be visible for non-authenticated users
		 */
		public function __construct() {
			$num = func_num_args();
			if ($num < 1 || $num > 6)
				throw new Exception('invalid argument number passed, must be either 1 or 6');
			
			$this->name = func_get_arg(0);
			assert(is_string($this->name));
			
			$tmp = ($num > 1) ? func_get_arg(1) : array();
			assert(is_array($tmp));
			foreach ($tmp as $alt)
				assert(is_string($alt));
			$this->alts = $tmp;
			
			$tmp = ($num > 2) ? func_get_arg(2) : array();
			assert(is_array($tmp));
			$this->users = array();
			foreach ($tmp as $user => $alts) {
				$tmp2 = $this->parseAlts($alts);
				assert ($tmp2 !== false);
				$this->users[$user] = $tmp2;
			}
			
			$this->finished = ($num > 3) ? (bool)func_get_arg(3) : false;
			$this->multi = ($num > 4) ? (bool)func_get_arg(4) : false;
			$this->publicly = ($num > 5) ? (bool)func_get_arg(5) : false;
		}
		
		/**
		 * @param  alt the alternative to add
		 */
		public function addAlternative($alt) {
			if (array_search($alt, $this->alts) === false)
				array_push($this->alts, $alt);
		}
		
		/**
		 * @return the possible alternatives this voting offers as string-array
		 */
		public function getAlternatives() {
			return $this->alts;
		}
		
		/**
		 * @param alt the alternative to remove
		 */
		public function removeAlternative($alt) {
			self::array_remove($alt, $this->alts);
			foreach ($this->users as &$user) {
				self::array_remove($alt, $user);
 			}
		}
		
		private static function array_remove($what, &$where) {
			$pos = array_search($alt, $where);
			if ($pos === false) return;
			unset($this->alts[$pos]);
		}
		
		/**
		 * @return the name of this voting
		 */
		public function getName() {
			return $this->name;
		}
		
		/**
		 * @return the result of this vote as an array mapping the alternatives to an integer indicating
		 *         the number of users who have voted for this alternative
		 */
		public function getVotes() {
			$votes = array_fill_keys($this->alts, 0);
			foreach ($this->users as $vote) {
				foreach ($vote as $alt) {
					$votes[$alt]++;
				}
			}
			return $votes;
		}
		
		/**
		 * @return whether multiple alternatives may be selected for this voting
		 */
		public function isMulti() {
			return $this->multi;
		}
		
		public function setMulti($yes = false) {
			$this->multi = !empty($yes);
		}
		
		/**
		 * @return whether this voting is visible for non-authorized users
		 */
		public function isPublicly() {
			return $this->publicly;
		}
		
		public function setPublicly($yes = false) {
			$this->publicly = !empty($yes);
		}
		
		/**
		 * @return whether this voting has been finished and no new votes are being accepted
		 */
		public function isFinished() {
			return $this->finished;
		}
		
		public function setFinished($yes = false) {
			$this->finished = !empty($yes);
		}
		
		/**
		 * @param  user name of the user to check permissions for
		 * @return whether the user exists <b>and</b> and has not yet voted for the voting specified
		 */
		public function mayVote($user) {
			return !array_key_exists($user, $this->users);
		}
		
		/**
		 * set / overwrite the vote of a user
		 * @param user the user whose vote shall be set
		 * @param alts the selected alternatives. If this is a multi-voting, an array must be passed otherwise
		 *             a single value may be passed as well
		 * @param overwrite optional argument which raises an exception if not set to <code>true</code> and the
		 *                  submitted user has voted already
		 */
		public function setVote($user, $alts, $overwrite = false) {
			if (!$overwrite && $this->hasVoted($user)) return;
			if ($this->isFinished()) return;
			$alts = parseAlts($alts);
			if ($alts !== false) $this->users[$user] = $alts;
		}
		
		private function parseAlts($alts) {
			if (count($alts) > 1) {
 				if (!$this->isMulti()) throw new Exception('passed multiple alternatives although voting is not multi');
				return array_intersect($this->alts, $alts);
			} elseif (is_array($alts)) {
				return array_intersect($this->alts, $alts);
			} elseif (array_search($alts, $this->alts) !== false) {
				return array($alts);
			} else {
				return false;
			}
		}
	}
	
?>